/*

Print a binary tree in the format defined by the dot program.

The dot program is part of the graphics visualization tool set that
is available from ATT.  <http://www.research.att.com/sw/tools/graphviz/>

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "reporter.h"
#include "tree.h"
#include "plottree.h"
#include "privatetree.h"

#define MAX_LABEL_LENGTH 100

void plotTreeHeader(SearchTree T, char *title) {

  char *author;			/* program author */
  time_t t;			/* time of day */
  char *timestring;
  char plotLabel[MAX_LABEL_LENGTH+1]; /* label string */

  char *attributes = "size=\"10,7.5\";\ncenter=true;\nnode [shape=record];\n";

  author = getTreeAuthor();

  t = time(NULL);
  timestring = ctime(&t);
  timestring[strlen(timestring)-1] = '\0';

  plotLabel[0] = '\0';
  if (title != NULL)
    strncat(plotLabel,title,MAX_LABEL_LENGTH-strlen(plotLabel));
  else
    strncat(plotLabel,"Binary Tree",MAX_LABEL_LENGTH-strlen(plotLabel));
  strncat(plotLabel,"\\n",MAX_LABEL_LENGTH-strlen(plotLabel));
  strncat(plotLabel,timestring,MAX_LABEL_LENGTH-strlen(plotLabel));
  strncat(plotLabel," - ",MAX_LABEL_LENGTH-strlen(plotLabel));
  strncat(plotLabel,author,MAX_LABEL_LENGTH-strlen(plotLabel));

  printf("/* Binary tree plot description */\n");
  printf("digraph %s {\n","G");
  printf("label=\"%s\";\n",plotLabel);
  printf("%s",attributes);
}

void plotTreeFooter(SearchTree T) {
  printf("} /* Tree height %i */\n",T->Height);
}

/*

Print the lines that define the nodes in the tree.

*/
void plotTreeNodes(SearchTree T, ElementPrintLabel P) {
  
  if (T == NULL) return;

  if (T->Left != NULL) plotTreeNodes(T->Left,P);

  printf("n%p[ label = \"<f0>.| <f1> ",T);
  (*(P))( T->Element);
  printf("|<f2>.\" ];");
  printf("\t/* height: %i",T->Height);
  printf(" */\n");

  if (T->Right != NULL) plotTreeNodes(T->Right,P);

}

/*

Print the lines that define links in the tree.

*/
void plotTreeEdges(SearchTree T) {
  
  if (T == NULL) return;

  if (T->Left != NULL) {
    printf("\"n%p\":f0 -> \"n%p\":f1;\n",T,T->Left);
    plotTreeEdges(T->Left);
  }
  
  if (T->Right != NULL) {
    printf("\"n%p\":f2 -> \"n%p\":f1;\n",T,T->Right);
    plotTreeEdges(T->Right);
  }
}
